@@ -129,6 +129,30 @@ static struct et8ek8_gain {
 	{256, 1023},  /* x16 */
 };
 
+static unsigned char et8ek8_exact_gain_to_gain(s32 g) {
+		static unsigned char lut[] = {0, 2, 3, 5, 6, 7, 8, 9,
+									  10, 11, 12, 12, 13, 14, 15, 15,
+									  16, 16, 17, 18, 18, 19, 19, 20,
+									  20, 20, 21, 21, 22, 22, 22, 23,
+									  23, 24, 24, 24, 25, 25, 25, 26,
+									  26, 26, 26, 27, 27, 27, 28, 28,
+									  28, 28, 29, 29, 29, 29, 30, 30, 
+									  30, 30, 30, 31, 31, 31, 31, 31,
+									  32, 32, 32, 32, 32, 33, 33, 33, 
+									  33, 33, 34, 34, 34, 34, 34, 34, 
+									  35, 35, 35, 35, 35, 35, 36, 36, 
+									  36, 36, 36, 36, 36, 37, 37, 37, 
+									  37, 37, 37, 37, 38, 38, 38, 38,
+									  38, 38, 38, 38, 39, 39, 39, 39, 
+									  39, 39, 39, 39, 40, 40, 40, 40};
+		
+        // Drop the bottom two bits and subtract 8
+        g = (g >> 2) - 8;
+        if (g < 0) return 0;
+        if (g >= 120) return 40;
+        return lut[g];
+}
+
 /* Called to change the V4L2 gain control value. This function
  * rounds and clamps the given value and updates the V4L2 control value.
  * If power is on, also updates the sensor analog and digital gains.
@@ -206,6 +230,9 @@ static int et8ek8_set_gain_exact(struct et8ek8_sensor *sensor, s32 gain)
 		digital = 0xc000 | gain;
 	}
 
+    // Set the (roughly) equivalent gain
+    sensor->controls[CTRL_GAIN].value = et8ek8_exact_gain_to_gain(gain);
+
         // FIXME: optimise I2C writes! 
 	r = smia_i2c_write_reg(sensor->i2c_client, SMIA_REG_8BIT,
 				  0x124a, analog >> 8);
@@ -282,7 +309,7 @@ static int et8ek8_set_frame_time(struct et8ek8_sensor *sensor, s32 frame_time)
 	chunk_time = (chunk_time + (clock >> 1)) / clock;
 	chunks = ((frame_time << 8) + (chunk_time >> 1)) / chunk_time;
 
-	//printk(KERN_INFO "Setting frame time to %d us (%d chunks)\n", frame_time, chunks);
+	// printk(KERN_INFO "Setting frame time to %d us (%d chunks)\n", frame_time, chunks);
 	if (chunks < 84) chunks = 84;
 	if (chunks > 2730) chunks = 2730;
 
@@ -435,8 +462,8 @@ static int et8ek8_update_controls(struct v4l2_int_device *s)
 			sensor->controls[CTRL_EXPOSURE].maximum;
 	}
 
-
-	/*printk(KERN_INFO
+	/*
+	printk(KERN_INFO
 	       "Updating et8ek8 controls for new mode:\n"
 	       " Width: %d\n"
 	       " Height: %d\n"
